// RELATED SOFTWARE: Scilab >= 5.5.0
// PACKAGE  : animaGIF : https://atoms.scilab.org/toolboxes/animaGIF
//            builds an animated GIF from png | jpg | bmp images
//  (stand-alone Scilab function. No external thirdparty application required).
// VERSION  : 1.0 - 2019-07-22
// AUTHOR   : Samuel GOUGEON
//
// DISCLAIMER: This toolbox is provided as is, with NO WARRANTY of any kind.
//             Use it to your own risks & responsability.
//

// ADAPTED FROM https://memorynotfound.com/generate-gif-image-java-delay-infinite-loop-example/

//package com.memorynotfound.image;

import javax.imageio.*;
import javax.imageio.metadata.IIOInvalidTreeException;
import javax.imageio.metadata.IIOMetadata;
import javax.imageio.metadata.IIOMetadataNode;
import javax.imageio.stream.ImageOutputStream;
import java.awt.image.RenderedImage;
import java.io.IOException;

public class GifSequenceWriter {

    protected ImageWriter writer;
    protected ImageWriteParam params;
    protected IIOMetadata metadata;

    public GifSequenceWriter(ImageOutputStream out, int imageType, int delay, int loop) throws IOException {
        writer = ImageIO.getImageWritersBySuffix("gif").next();
        params = writer.getDefaultWriteParam();

        ImageTypeSpecifier imageTypeSpecifier = ImageTypeSpecifier.createFromBufferedImageType(imageType);
        metadata = writer.getDefaultImageMetadata(imageTypeSpecifier, params);

        configureRootMetadata(delay, loop);

        writer.setOutput(out);
        writer.prepareWriteSequence(null);
    }

    private void configureRootMetadata(int delay, int loop) throws IIOInvalidTreeException {
        String metaFormatName = metadata.getNativeMetadataFormatName();
        IIOMetadataNode root = (IIOMetadataNode) metadata.getAsTree(metaFormatName);
        /*
        // LogicalScreenDescriptor  (added)
        IIOMetadataNode logicalScreenNode = getNode(root, "LogicalScreenDescriptor");
        logicalScreenNode.setAttribute("GlobalColorTableFlag", "FALSE");
        logicalScreenNode.setAttribute("ColorResolution", "1");
        logicalScreenNode.setAttribute("SortFlag", "FALSE");
        logicalScreenNode.setAttribute("GlobalColorTableSize", "0");
        */

        IIOMetadataNode graphicsControlExtensionNode = getNode(root, "GraphicControlExtension");
        graphicsControlExtensionNode.setAttribute("disposalMethod", "none");
        graphicsControlExtensionNode.setAttribute("userInputFlag", "FALSE");
        graphicsControlExtensionNode.setAttribute("transparentColorFlag", "FALSE");
        graphicsControlExtensionNode.setAttribute("delayTime", Integer.toString(delay / 10));
        graphicsControlExtensionNode.setAttribute("transparentColorIndex", "0");

//        IIOMetadataNode commentsNode = getNode(root, "CommentExtensions");
//        commentsNode.setAttribute("CommentExtension", "Created by: https://atoms.scilab.org/toolboxes/animatedGIF for Scilab");

        IIOMetadataNode appExtensionsNode = getNode(root, "ApplicationExtensions");
        IIOMetadataNode child = new IIOMetadataNode("ApplicationExtension");
        child.setAttribute("applicationID", "NETSCAPE"); // Mandatory
        child.setAttribute("authenticationCode", "2.0");
        child.setUserObject(new byte[]{1, (byte) loop, (byte) 0});
        appExtensionsNode.appendChild(child);
        metadata.setFromTree(metaFormatName, root);
    }

    private static IIOMetadataNode getNode(IIOMetadataNode rootNode, String nodeName){
        int nNodes = rootNode.getLength();
        for (int i = 0; i < nNodes; i++){
            if (rootNode.item(i).getNodeName().equalsIgnoreCase(nodeName)){
                return (IIOMetadataNode) rootNode.item(i);
            }
        }
        IIOMetadataNode node = new IIOMetadataNode(nodeName);
        rootNode.appendChild(node);
        return(node);
    }

    public void writeToSequence(RenderedImage img) throws IOException {
        writer.writeToSequence(new IIOImage(img, null, metadata), params);
    }

    public void close() throws IOException {
        writer.endWriteSequence();
    }
}
