// RELATED SOFTWARE: Scilab >= 5.5.0
// PACKAGE  : animaGIF : https://atoms.scilab.org/toolboxes/animaGIF
//            builds an animated GIF from png | jpg | bmp images
//  (stand-alone Scilab function. No external thirdparty application required).
// VERSION  : 1.0 - 2019-07-22
// AUTHOR   : Samuel GOUGEON
// COPYRIGHT: Samuel GOUGEON -  2019
// LICENSE  : CeCILL : http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
//
// DISCLAIMER: This toolbox is provided as is, with NO WARRANTY of any kind.
//             Use it to your own risks & responsability.
//

function idGif = animaGIF(varargin)
    // animaGIF(filemask, outgif)
    // animaGIF(imagefiles, outgif)
    // animaGIF(.., delay)
    // animaGIF(.., delay, loops)
    //
    // animaGIF(idFig, outgif)
    // animaGIF(idFig, outgif, delay)
    // animaGIF(idFig, outgif, delay, loops)
    // animaGIF(idFig, idGif)
    // animaGIF(idGif)

    fname = "animaGIF"
    idGif = []
    addMode = %f    // Is it a animaGIF(idFig, idGif) call?

    // Testing the Scilab mode
    if and(getscilabmode()<>["STD" "NW"]) then
        msg = _(fname,"%s can be run only in STD or NW Scilab modes.\n")
        error(msprintf(msg, fname))
    end
    // Checking the number of input arguments
    if length(varargin) < 1 | length(varargin) > 4 then
        msg = _("%s: Wrong number of input arguments: %d to %d expected.\n")
        error(msprintf(msg, fname, 1, 4))
    end

    // PROCESSING SIMPLE CASES
    // =======================
    tmp = varargin(1)

    // animaGIF(idGif)
    // ---------------
    if typeof(tmp)=="_EObj" & jgetclassname(tmp)=="GifSequenceWriter" then
        tmp.close()
        global animaGIF_output_handle
        animaGIF_output_handle.close()
        jremove tmp animaGIF_output_handle
        deletefile(TMPDIR + "/animaGIF_out.png")
        return
    end
    // animaGIF(idFig,..)
    // ------------------
    if type(tmp)==9
        if size(tmp,"*")>1 then
            msg = _("%s: Argument #%d: Scalar (1 element) expected.\n")
            error(msprintf(msg, fname, 1))
        end
        if tmp.type <> "Figure" then
            msg = _("%s: Argument #%d: Graphic handle of type ''%s'' expected.\n")
            error(msprintf(msg, fname, 1, "Figure"))
        end
        if length(varargin)==1 then
            msg = _("%s: Wrong number of input arguments: %d to %d expected.\n")
            error(msprintf(msg, fname, 2, 4))
        end
        varargin(1) = null()
        imagefiles = TMPDIR + "/animaGIF_out.png"
        xs2png(tmp, imagefiles)
        idFig = tmp
        tmp = varargin(1)   // idGif | outgif
        if typeof(tmp)=="_EObj" & jgetclassname(tmp)=="GifSequenceWriter" then
            addMode = %t
            idGif = tmp
        elseif type(tmp) <> 10
            msg = _(fname, "%s: Argument #%d: Text or Java GifSequenceWriter object expected.\n")
            error(msprintf(msg, fname, 2))
        else
            outgif = tmp(1)
        end

    elseif type(tmp)==10
        // animaGIF(filemask, ..) | animaGIF(imagefiles, ..)
        // -------------------------------------------------
        if length(varargin)==1 then
            msg = _("%s: Wrong number of input arguments: %d to %d expected.\n")
            error(msprintf(msg, fname, 2, 4))
        end
        imagefiles = tmp
        varargin(1) = null();
        mask = []
        if grep(imagefiles(1),"*") <> [] then
            if size(imagefiles,"*")==1
                mask = stripblanks(imagefiles)
                // We check that the directory exists:
                tmp = fileparts(mask)
                if mask=="" | (tmp<>"" & ~isdir(tmp))
                    msg = _(fname, "%s: Argument #%d: The directory ''%s'' does not exist.\n")
                    error(msprintf(msg, fname, 1, tmp))
                end
                imagefiles = listfiles(mask)
            else
                mask = stripblanks(imagefiles(1))
                imagefiles(1) = []
                if mask == ""
                    mask = []
                end
            end
        end
        if imagefiles==[] then
            warning(msprintf(_(fname,"%s: No images selected.\n"), fname))
            return
        end
        if mask <> []
            [?,mask] = fileparts(mask)
            imagefiles = animaGIF_numsort(imagefiles, mask)
        end
        //
        outgif = varargin(1)
        if type(outgif) <> 10
            msg = _(fname, "%s: Argument #%d: Text expected.\n")
            error(msprintf(msg, fname, 2))
        end
        outgif = outgif(1)
    else
        msg = _(fname, "%s: Argument #%d: Text or Graphic handle or Java GifSequenceWriter object expected.\n")
        error(msprintf(msg, fname, 1))
    end

    if  ~addMode
        // outgif
        // -------
        // Checking done here-above
        varargin(1) = null()

        // Delay
        // -----
        delay0 = 200
        if length(varargin)>0 then
            if type(varargin(1))==0
                delay = delay0
            else
                delay = varargin(1)
                delay = delay(1)
                if type(delay)==10
                    delay = evstr(delay)
                end
                if type(delay) <> 1 | ~isreal(delay,0) then
                    msg = _("%s: Argument #%d: Decimal number expected.\n")
                    error(msprintf(msg, fname, 3))
                end
                delay = real(delay)
                if delay <= 0 then
                    msg = _("%s: Argument #%d: Must be > %d.\n")
                    error(msprintf(msg, fname, 3, 0))
                end
            end
            varargin(1) = null()
        else
            delay = delay0
        end

        // Looping
        // -------
        if length(varargin)>0 then
            loops = varargin(1)
            varargin(1) = null()
            if type(loops)==10
                loops = evstr(convstr(loops(1)))
            end
            if type(loops) <> 1 | ~isreal(loops,0) then
                msg = _("%s: Argument #%d: Decimal number expected.\n")
                error(msprintf(msg, fname, 4))
            end
            loops = real(loops(1))
            if loops <> 0 & (loops < 1.5 | loops >= 255.5) then
                msg = _("%s: Argument #%d: Must be in the interval [%d, %d].\n")
                error(msprintf(msg, fname, 4, 2, 255))
            end
            loops = round(loops)-1
        else
            loops = 0 // infinite looping
        end
    end     // ~addMode

    // Java initializations
    // ====================
    global ImageIO FileImageOutputStream ImageOutputStream BufferedImage File
    if ImageIO==[] | ~jexists(ImageIO) | File==[] | ~jexists(File)
        // mprintf("jimports\n")
        clearglobal ImageIO FileImageOutputStream ImageOutputStream BufferedImage File
        clear ImageIO FileImageOutputStream ImageOutputStream BufferedImage File
        jimport javax.imageio.ImageIO;
        jimport javax.imageio.stream.FileImageOutputStream;
        jimport javax.imageio.stream.ImageOutputStream;
        jimport java.awt.image.BufferedImage;
        jimport java.io.File;
        global ImageIO FileImageOutputStream ImageOutputStream BufferedImage File
    end

    global GifSequenceWriter
    if GifSequenceWriter==[] | ~jexists(GifSequenceWriter) then
        // mprintf("jcompile\n")
        clearglobal GifSequenceWriter
        clear GifSequenceWriter
        javacodefile = animaGIFpath() + "macros/GifSequenceWriter.java";
        t = mgetl(javacodefile);
        GifSequenceWriter = jcompile("GifSequenceWriter", t);
        global GifSequenceWriter
    end

    // PROCESSING
    // ==========
    // First image
    template = ImageIO.read(File.new(imagefiles(1)));
    if ~addMode then
        output = FileImageOutputStream.new(File.new(pathconvert(outgif,%f)));
        idGif = GifSequenceWriter.new(output, template.getType(), delay, loops);
    end
    idGif.writeToSequence(template);

    fichiers = imagefiles(2:$);

    // Waitbar initialization
    i = 1
    n = size(imagefiles,"*")
    if n>2 then
        msg = msprintf(_(fname,"Building the animated GIF (%d images)..."),n)
        idBar = waitbar(i/n, msg)
    end

    // Remaining images
    for fichier = fichiers(:)'
        image = File.new(fichier);
        next = ImageIO.read(image);
        idGif.writeToSequence(next);
        i = i + 1
        waitbar(i/n, idBar)
    end

    // Closing the waitbar
    if isdef("idBar","l") then
        close(idBar)
    end

    // Closing the stream
    if ~isdef("idFig","l") then
        idGif.close();
        output.close();
        jremove idGif output
        idGif = []
    elseif ~addMode // = creation
        // We save the output stream handle to properly close it at the end
        global animaGIF_output_handle
        animaGIF_output_handle = output 
    end
    jremove template image next
endfunction

// ========================================================

function T = animaGIF_numsort(T, mask)
    mask = strsubst(mask, "\", "\\")
    mask = strsubst(mask, "*", "(\d+)")
    mask = strsubst(mask, ":", "\:")
    mask = strsubst(mask, ".", "\.")
    mask = strsubst(mask, "-", "\-")
    r = []
    for t = basename(T(:)')
        [iD,iF,match,samp] = regexp(t, "|"+mask+"|", "o")
        if samp <> []
            r = [r samp]
        else
            r = [r "-1"]
        end
    end
    r = evstr(r)
    T(r==-1) = []
    r(r==-1) = []
    [?,k] = gsort(r,"g","i");
    T = T(k)
endfunction
