// RELATED SOFTWARE: Scilab >= 5.5.0
// PACKAGE  : animaGIF() : https://atoms.scilab.org/toolboxes/animaGIF
// VERSION  : 1.0 - 2019-07-22
// AUTHOR   : Samuel GOUGEON - private individual
// COPYRIGHT: Samuel GOUGEON -  2019
// LICENSE  : CeCILL : http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
//
// ==========================================================================
// Unitary tests for animaGIF()
// ============================

// Data for the figure
// -------------------
u = linspace(0,%pi,40);
v = linspace(0,2*%pi,40);
[u,v] = meshgrid(u,v);
k = 0.10;
r = k*(2 + sin(2*u) + sin(4*u)/2);
x = sin(2*u)/2 - sin(4*u)/4 + r.*cos(v).*cos(u-sin(2*u))*1.5;
y = cos(2*u) - r.*cos(v).*sin(u-sin(2*u));
z = r.*sin(v);

//
Dir = TMPDIR + "/animaGIF/";
mkdir(Dir);

// Output filename
// ---------------
fichier = Dir + "anima.gif";
deletefile(fichier);
tmp = ~isfile(fichier);
clf reset
surf(y, z, x)
f = gcf();
id = animaGIF(f, fichier);
animaGIF(id);
assert_checktrue(tmp & isfile(fichier));

// Error messages
// --------------
fname = "animaGIF";
f = gcf();

msg = msprintf(_("%s: Wrong number of input arguments: %d to %d expected.\n"), fname, 1, 4);
assert_checkerror("animaGIF();", msg);
assert_checkerror("animaGIF(1,1,1,1,1);", msg);
msg = msprintf(_("%s: Wrong number of input arguments: %d to %d expected.\n"), fname, 2, 4);
assert_checkerror("animaGIF(""filemask"");", msg);
assert_checkerror("animaGIF(f);", msg);
// argin(1)
msg = _(fname, "%s: Argument #%d: Text or Graphic handle or Java GifSequenceWriter object expected.\n")
msg = msprintf(msg, fname, 1);
assert_checkerror("animaGIF(%t);", msg);
assert_checkerror("animaGIF(1);", msg);
assert_checkerror("animaGIF(int8(1));", msg);
// idFig
msg = msprintf(_("%s: Argument #%d: Scalar (1 element) expected.\n"), fname, 1);
assert_checkerror("animaGIF([f f], fichier);", msg);

msg = _("%s: Argument #%d: Graphic handle of type ''%s'' expected.\n")
msg = msprintf(msg, fname, 1, "Figure");
assert_checkerror("animaGIF(gca(), fichier);", msg);
// idFig, outgif | idGif
msg = _(fname, "%s: Argument #%d: Text or Java GifSequenceWriter object expected.\n")
msg = msprintf(msg, fname, 2);
assert_checkerror("animaGIF(f, %t);", msg);
assert_checkerror("animaGIF(f, jwrap(%t));", msg);

// outgif 
mask = ascii(grand(1,10,"uin",ascii("a"),ascii("z")))+"/*.png";
msg = _(fname, "%s: Argument #%d: The directory ''%s'' does not exist.\n");
msg = msprintf(msg, fname, 1, fileparts(mask));
assert_checkerror("animaGIF(mask,""out.gif"");", msg);

msg = msprintf(_(fname, "%s: Argument #%d: Text expected.\n"), fname, 2);
mask = animaGIFpath()+"tests/images/*.png";
assert_checkerror("animaGIF(mask, %t);", msg);
//Delay
msg = msprintf(_("%s: Argument #%d: Decimal number expected.\n"), fname, 3);
assert_checkerror("animaGIF(mask, fichier, %t)", msg);
assert_checkerror("animaGIF(mask, fichier, %i)", msg);
msg = msprintf(_("%s: Argument #%d: Must be > %d.\n"), fname, 3, 0);
assert_checkerror("animaGIF(mask, fichier, -10)", msg);
// Loops
msg = msprintf(_("%s: Argument #%d: Decimal number expected.\n"), fname, 4);
assert_checkerror("animaGIF(mask, fichier, , %t)", msg);
assert_checkerror("animaGIF(mask, fichier, , %i)", msg);
msg = msprintf(_("%s: Argument #%d: Must be in the interval [%d, %d].\n"), fname, 4, 2, 255);
assert_checkerror("animaGIF(mask, fichier, , 1)", msg);
