// RELATED SOFTWARE: Scilab >= 5.5.0
// PACKAGE  : animaGIF() : https://atoms.scilab.org/toolboxes/animaGIF
// VERSION  : 1.0 - 2019-07-22
// AUTHOR   : Samuel GOUGEON
// COPYRIGHT: Samuel GOUGEON -  2019
// LICENSE  : CeCILL : http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
//
// <-- INTERACTIVE TEST -->
// ===========================================================================
// Unitary INTERACTIVE tests for animaGIF()
// ========================================
//
Dir = TMPDIR + "/animaGIF/";
apath = animaGIFpath();
mkdir(Dir);

// Images files selection, images encodings, delay and looping
// -----------------------------------------------------------
// CHECK:
//  - The animation exists = JPG images accepted
//  - image = vertical colored bottle
//  - soft rotation (36 images)
//  - 2 iterations
//  - Duration before stopping = 14.7 s (default delay = 200 ms)
//    Click on the STOP button to stop the chronometer when the GIF stops.
// GO:
outgif = Dir+"animaV.gif";
animaGIF(apath+"tests/images/animaV_*.jpg", outgif, , 2);
animaGIFinHTML(outgif, %t);


// CHECK: 
//  - The animation exists = PNG images accepted
// - Roman surface (tetraedra with rounded edges and crunched faces)
// - soft rotation (36 images)
// - 3 full rotations
// - rotation twice slower => 43.7 s
// GO:
outgif = Dir+"romanSurf.gif";
animaGIF(apath+"tests/images/romanSurf_*.png", outgif, 400, 3);
animaGIFinHTML(outgif, %t);


// CHECK: 
// - Mesh white bottle (soft rotation : 36 images)
// - rotations never stop
// - rotation speed (with default delay) => 3 tours = 21.6 s
// GO:
outgif = Dir+"animaMesh.gif";
animaGIF(apath+"tests/images/animaMesh_*.jpg", outgif);
animaGIFinHTML(outgif, %t);


// Mixing input images of different encodings. Various input images sizes
// ----------------------------------------------------------------------
// CHECK: 
// - The animation is well yielded and runs.
// - The 3 sub-series coming from 3 distinct encodings exist.
// - Rendering in the web browser: the first image sets the size of the whole
//   animated GIF. Changes in input size lead to clipping and remaining margins.
// - Rendering in an image viewer software: It may depend on the softawe.
//   In IrfanView 4.53: each of the 3 sequences is displayed in its full size,
//   but extra-margins remain from previous wide sequences.
// GO:
Dir = TMPDIR + "/animaGIF/"; mkdir(Dir);
outgif = Dir + "animix.gif";
apath = animaGIFpath();
L1 = listfiles(apath+"tests/images/animaMesh_*.jpg");
L2 = listfiles(apath+"tests/images/romanSurf_*.png");
L3 = listfiles(apath+"tests/images/animaV_*.bmp");
L = [L1(1:$/3) ; L2(2*$/3:$) ; L3];
assert_checkequal(size(L), [38 1]);
animaGIF(L, outgif, 300, 3);
animaGIFinHTML(outgif); // in the web browser
winopen(outgif);        // in the dedicated image viewer software


// Numerical re-sorting of input files
// -----------------------------------
Dir = TMPDIR + "/animaGIF/"; mkdir(Dir);
outgif = Dir + "animix.gif";
apath = animaGIFpath();
L1 = listfiles(apath+"tests/images/animaMesh_*.jpg");
L2 = listfiles(apath+"tests/images/animaH_*.png");
L = [L1(1:2:$) ; L2(2:2:$)];
messagebox(strsubst(L,apath,"./"))
// CHECK:
//  - There are 2 subsequences, one with the mesh, then the colored one.
//  - Each sequence goes twice faster, since one angle step over 2 is missed
//    (20°/step).
animaGIF(L, outgif, 300, 3);
animaGIFinHTML(outgif); // in the web browser

// Now we force numerical ordering:
L = ["*"; L];
animaGIF(L, outgif, 300, 3);
animaGIFinHTML(outgif); // in the web browser
// CHECK:
//  - The rotation is twice progressive/softer (normal 10° angle step)
//  - The two series are intricated: one mesh follows one colored, etc.

// Cleaning
rmdir(Dir,'s');
